# -*- coding:utf-8 -*-
# Author: hankcs
# Date: 2019-12-28 22:19

CTB6_CWS_HOME = 'http://file.hankcs.com/corpus/ctb6_cws.zip'

CTB6_CWS_TRAIN = 'http://file.hankcs.com/corpus/ctb6_cws.zip#train.txt'
CTB6_CWS_VALID = 'http://file.hankcs.com/corpus/ctb6_cws.zip#dev.txt'
CTB6_CWS_TEST = 'http://file.hankcs.com/corpus/ctb6_cws.zip#test.txt'
