# -*- coding:utf-8 -*-
# Author: hankcs
# Date: 2019-12-28 23:13

MSRA_NER_HOME = 'http://file.hankcs.com/corpus/msra_ner.zip'

MSRA_NER_TRAIN = f'{MSRA_NER_HOME}#train.tsv'
MSRA_NER_VALID = f'{MSRA_NER_HOME}#dev.tsv'
MSRA_NER_TEST = f'{MSRA_NER_HOME}#test.tsv'
