# -*- coding:utf-8 -*-
# Author: hankcs
# Date: 2019-12-05 23:05
from hanlp.components.taggers.cnn_tagger import CNNTagger
from hanlp.components.taggers.rnn_tagger import RNNTagger


class CNNPartOfSpeechTagger(CNNTagger):
    pass


class RNNPartOfSpeechTagger(RNNTagger):
    pass
